@@ -7,6 +7,7 @@
 
 class Crypt;
 class Cert;
+class Key;
 
 
 /**
@@ -19,12 +20,14 @@ class Hash : public RCObj
 private:
     HCRYPTHASH hhash;
     Crypt *parent;
-    void init(Crypt *ctx) throw(CSPException);
+    Key *pkey;
+    void init(Crypt *ctx, Key *key) throw(CSPException);
+    HMAC_INFO   HmacInfo;
 protected:
     //
 public:
-    Hash(Crypt *ctx, BYTE *STRING, DWORD LENGTH) throw(CSPException);
-    Hash(Crypt *ctx) throw(CSPException);
+    Hash(Crypt *ctx, BYTE *STRING, DWORD LENGTH, Key *key=0) throw(CSPException);
+    Hash(Crypt *ctx, Key *key=0) throw(CSPException);
 
     virtual ~Hash() throw(CSPException);
 
@@ -32,6 +35,7 @@ public:
     void update(BYTE *STRING, DWORD LENGTH) throw(CSPException);
     void sign(BYTE **s, DWORD *slen, DWORD dwKeyspec=AT_KEYEXCHANGE) throw(CSPException);
     bool verify(Cert *cert, BYTE *STRING, DWORD LENGTH) throw(CSPException);
+    Key *derive_key() throw(CSPException);
 
     friend class Crypt;
 };
